/*
 * File   : opCDDB.c
 * Purpose: This file contains the routines to set some options.
 * Autor  : Andreas Theofilu
 *
 * Copyright (C) 1999, 2000 by Theos Soft.
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <unistd.h>

#include <Xmt/Xmt.h>
#include <Xmt/Create.h>
#include <Xmt/Converters.h>
#include <Xmt/Xpm.h>
#include <Xmt/Procedures.h>
#include <Xmt/WidgetType.h>
#include <Xmt/Dialog.h>
#include <Xmt/Dialogs.h>
#include <Xmt/Chooser.h>
#include <Xbae/Matrix.h>
#include <cdaudio.h>
#include "platten.h"

typedef struct
{
	int remote;
	int proxy;
	char prserver[129];
	int prport;
	char cdpath[256];
	char mailcmd[128];
	char cdserver[128];
	char cdmail[128];
} opcddb_t;

XtResource opcddb_resource[] = {
	{"remote", "remote", XtRInt, sizeof (int),
	 XtOffsetOf (opcddb_t, remote), XtRImmediate, NULL},
	{"proxy", "proxy", XtRInt, sizeof (int),
	 XtOffsetOf (opcddb_t, proxy), XtRImmediate, NULL},
	{"prserver", "prserver", XmtRBuffer, XmtSizeOf (opcddb_t, prserver),
	 XtOffsetOf (opcddb_t, prserver), XtRImmediate, NULL},
	{"prport", "prport", XtRInt, sizeof (int),
	 XtOffsetOf (opcddb_t, prport), XtRImmediate, NULL},
	{"cdpath", "cdpath", XmtRBuffer, XmtSizeOf (opcddb_t, cdpath),
	 XtOffsetOf (opcddb_t, cdpath), XtRImmediate, NULL},
	{"mailcmd", "mailcmd", XmtRBuffer, XmtSizeOf (opcddb_t, mailcmd),
	 XtOffsetOf (opcddb_t, mailcmd), XtRImmediate, NULL},
	{"cdserver", "cdserver", XmtRBuffer, XmtSizeOf (opcddb_t, cdserver),
	 XtOffsetOf (opcddb_t, cdserver), XtRImmediate, NULL},
	{"cdmail", "cdmail", XmtRBuffer, XmtSizeOf (opcddb_t, cdmail),
	 XtOffsetOf (opcddb_t, cdmail), XtRImmediate, NULL}
};

/*
 * Einige globale Variablen
 */
opcddb_t opcddb;
Widget OpCDDBWidget = (Widget)0;	/* Main widget */

Widget wProxy;
Widget wPrserver;
Widget wPrport;
Widget wCdpath;
Widget wCdserver;
Widget wCdmail;
Widget wMailcmd;
Widget wLbserver;
Widget wLbmail;
Widget wCdplus;
Widget wCdminus;
Widget wMaplus;
Widget wMaminus;

/*
 * Globale Funktionsdeklarationen die nur in diesem Modul verwendet
 * werden.
 */
void OpCDDBDialogInit (Widget w);
int SplittServerLine (char *line, char *sname, int *sport, int *proto, char *addr);

/*
 * Handling der Dialogbox zur Eingabe von Singles
 */
void OpCDDB ()
{
struct cddb_conf conf;
struct cddb_serverlist list;
struct cddb_server proxy;
struct MAIL *akt;

	XmtDisplayBusyCursor (toplevel);
	memset ((void *)&opcddb, 0, sizeof (opcddb_t));

	if (cddb_read_serverlist (&conf, &list, &proxy) != 0)
	{
	   XmtDisplayError (toplevel, "ErrorOptionEnv", NULL);
	   XmtDisplayDefaultCursor (toplevel);
	   return;
	}

	if (!OpCDDBWidget)
	{
	   if (!GetResourcePath ("OpCDDB"))
	   {
	      XmtDisplayError (toplevel, "ErrorNoWidgets",
			"No Widgets for dialog window found or\nan error occured while loading the widgets!");
	      XmtDisplayDefaultCursor (toplevel);
	      return;
	   }

	   OpCDDBWidget = XmtBuildQueryDialog (toplevel, "opcddb", 
		opcddb_resource, XtNumber (opcddb_resource),
		"proxy", &wProxy, "prserver", &wPrserver,
		"prport", &wPrport, "cdpath", &wCdpath,
		"mailcmd", &wMailcmd, "cdserver", &wCdserver,
		"cdmail", &wCdmail, "lbserver", &wLbserver,
		"lbmail", &wLbmail, "cdplus", &wCdplus,
		"cdminus", &wCdminus, "maplus", &wMaplus,
		"maminus", &wMaminus, NULL);

	   XmtAddDeleteCallback (XmtGetShell (OpCDDBWidget), XmDO_NOTHING,
		XmtDialogCancelCallback, NULL);
	}

	/*
	 * Auswertung der Daten welche aus der Konfigurationsdatei
	 * ausgelesen wurden.
	 */
	if (conf.conf_access == CDDB_ACCESS_REMOTE)
	{
	   char hv0[128], *proto;
	   int i;
	   String *row;

	   opcddb.remote = 1;

	   if (conf.conf_proxy == CDDB_PROXY_ENABLED)
	   {
	      opcddb.proxy = 1;
	      strcpy (opcddb.prserver, proxy.server_name);
	      opcddb.prport = proxy.server_port;
	   }

	   /*
	    * Die folgenden Zeilen fuellen die linke Listbox des Fensters
	    * mit den CDDB-Servern auf. Die Server wurden von der Library
	    * libcdaudio geliefert.
	    */
	   XbaeMatrixDisableRedisplay (wLbserver);
	   XbaeMatrixDeleteRows (wLbserver, 0, XbaeMatrixNumRows (wLbserver));

	   for (i = 0; i < list.list_len; i++)
	   {
	      switch (list.list_host[i].host_protocol)
	      {
		 case CDDB_MODE_HTTP:  proto = "HTTP"; break;
		 case CDDB_MODE_CDDBP: proto = "CDDBP"; break;
	         default:              proto = "HTTP"; break;
	      }

	      sprintf (hv0, "%s %d %s %s",
	         list.list_host[i].host_server.server_name,
	         list.list_host[i].host_server.server_port, proto,
	         list.list_host[i].host_addressing);
	      row = (String *)XtMalloc (sizeof (String));
	      row[0] = &hv0[0];
	      XbaeMatrixAddRows (wLbserver, i, row, NULL, NULL, 1);
	      XtFree ((char *)row);
	   }

	   XbaeMatrixEnableRedisplay (wLbserver, True);
	   strcpy (opcddb.cdpath, option.ipCDDBPath);
	   strcpy (opcddb.mailcmd, option.ipMailCmd);

	   /*
	    * Die folgenden Zeilen fuellen die rechte Listbox des Fensters
	    * mit E-Mailadressen auf. Die Mailadressen kommen aus der
	    * lokalen Optionendatei und wurden beim Start des Programms
	    * in eine dynamische Struktur eingelesen.
	    */
	   if (firstMail != NULL)
	   {
	      String *row;
	      int i = 0;

	      XbaeMatrixDisableRedisplay (wLbmail);
	      XbaeMatrixDeleteRows (wLbmail, 0, XbaeMatrixNumRows (wLbmail));
	      akt = firstMail;

	      while (akt)
	      {
	         row = (String *)XtMalloc (sizeof (String));
	         row[0] = akt->mail;
	         XbaeMatrixAddRows (wLbmail, i, row, NULL, NULL, 1);
	         XtFree ((char *)row);
		 akt = akt->next;
		 i++;
	      }

	      XbaeMatrixEnableRedisplay (wLbmail, True);
	   }
	}

	OpCDDBDialogInit (OpCDDBWidget);

	if (XmtDialogDoSync (OpCDDBWidget, &opcddb))
	{
	   int i, j;
	   String s;

	   if (opcddb.remote)
	   {
	      conf.conf_access = CDDB_ACCESS_REMOTE;

	      if (opcddb.proxy && strlen (opcddb.prserver) &&
		  opcddb.prport > 0 && opcddb.prport < 32767)
	      {
	         conf.conf_proxy = CDDB_PROXY_ENABLED;
		 strcpy  (proxy.server_name, opcddb.prserver);
		 proxy.server_port = opcddb.prport;
	      }
	      else
		 conf.conf_proxy = CDDB_PROXY_DISABLED;

	      list.list_len = XbaeMatrixNumRows (wLbserver);
	      i = 0;

	      while (i < list.list_len)
	      {
	         s = XbaeMatrixGetCell (wLbserver, i, 0);
		 SplittServerLine (s,
			&list.list_host[i].host_server.server_name[0],
			&list.list_host[i].host_server.server_port,
			&list.list_host[i].host_protocol,
			&list.list_host[i].host_addressing[0]);
		 i++;
	      }
	   }
	   else
	      conf.conf_access = CDDB_ACCESS_LOCAL;

	   cddb_write_serverlist (conf, list, proxy);
	   strcpy (option.ipCDDBPath, opcddb.cdpath);
	   strcpy (option.ipMailCmd, opcddb.mailcmd);

	   /*
	    * Nun noch die rechte Listbox ausleeren und speichern
	    */
	   if (firstMail)
	   {
	      akt = firstMail;

	      while (akt)
	      {
		 akt = akt->next;
		 XtFree ((char *)firstMail);
		 firstMail = akt;
	      }

	      firstMail = NULL;
	   }

	   j = XbaeMatrixNumRows (wLbmail);
	   i = 0;
	   akt = firstMail;

	   while (i < j)
	   {
	      s = XbaeMatrixGetCell (wLbmail, i, 0);
   
	      if (!firstMail)
	      {
		 if ((firstMail = (struct MAIL *)XtMalloc (sizeof (struct MAIL))) == NULL)
		 {
		    XmtDisplayError (toplevel, "ErrorNoMem", NULL);
		    XmtDisplayDefaultCursor (toplevel);
		    return;
		 }

		 akt = firstMail;
		 akt->next = NULL;
	      }
	      else
	      {
		 struct MAIL *new;

		 if ((new = (struct MAIL *)XtMalloc (sizeof (struct MAIL))) == NULL)
		 {
		    XmtDisplayError (toplevel, "ErrorNoMem", NULL);
		    XmtDisplayDefaultCursor (toplevel);
		    return;
		 }

		 akt->next = new;
		 new->next = NULL;
		 akt = new;
	      }

	      strcpy (akt->mail, s);
	      i++;
	   }

	   WriteOptions ();
	}

	XmtDisplayDefaultCursor (toplevel);
}

void OpCDDBDialogInit (Widget w)
{
	XtSetSensitive (wProxy, (opcddb.remote) ? True : False);
	XtSetSensitive (wPrserver, (opcddb.remote && opcddb.proxy) ? True : False);
	XtSetSensitive (wPrport, (opcddb.remote && opcddb.proxy) ? True : False);
	XtSetSensitive (wCdpath, (opcddb.remote) ? True : False);
	XtSetSensitive (wCdserver, (opcddb.remote) ? True : False);
	XtSetSensitive (wCdmail, (opcddb.remote) ? True : False);
	XtSetSensitive (wMailcmd, (opcddb.remote) ? True : False);
	XtSetSensitive (wLbserver, (opcddb.remote) ? True : False);
	XtSetSensitive (wLbmail, (opcddb.remote) ? True : False);
	XtSetSensitive (wCdplus, (opcddb.remote) ? True : False);
	XtSetSensitive (wCdminus, (opcddb.remote) ? True : False);
	XtSetSensitive (wMaplus, (opcddb.remote) ? True : False);
	XtSetSensitive (wMaminus, (opcddb.remote) ? True : False);
}

void opRemoteChooserCallback (Widget w, XtPointer tag, XtPointer data)
{
	XmtDialogGetDialogValues (OpCDDBWidget, &opcddb);
	OpCDDBDialogInit (w);
}

void opProxyChooserCallback (Widget w, XtPointer tag, XtPointer data)
{
	XmtDialogGetDialogValues (OpCDDBWidget, &opcddb);
	OpCDDBDialogInit (w);
}

/*
 * Folgende Funktion wird aufgerufen wenn der Button '+' ueber der
 * linke Listbox gedrueckt wird. Die Funktion fuegt den Inhalt der
 * Eingabezeile (links vom Button) in die linke Listbox ein.
 */
void AddCDDBServer (Widget w, XtPointer tag, XtPointer data)
{
int pos;
String *row;

	XmtDialogGetDialogValues (OpCDDBWidget, &opcddb);

	if (!strlen (opcddb.cdserver))
	   return;

	XbaeMatrixDisableRedisplay (wLbserver);
	pos = XbaeMatrixNumRows (wLbserver);
	row = (String *)XtMalloc (sizeof (String));
	row[0] = opcddb.cdserver;
	XbaeMatrixAddRows (wLbserver, pos, row, NULL, NULL, 1);
	XtFree ((char *)row);
	XbaeMatrixEnableRedisplay (wLbserver, True);
}

/*
 * Folgende Funktion wird aufgerufen wenn der Button '+' ueber der
 * rechten Listbox gedrueckt wird. Die Funktion fuegt den Inhalt der
 * Eingabezeile (links vom Button) in die rechte Listbox ein.
 */
void AddEMAILDestination (Widget w, XtPointer tag, XtPointer data)
{
int pos;
String *row;

	XmtDialogGetDialogValues (OpCDDBWidget, &opcddb);

	if (!strlen (opcddb.cdmail))
	   return;

	XbaeMatrixDisableRedisplay (wLbmail);
	pos = XbaeMatrixNumRows (wLbmail);
	row = (String *)XtMalloc (sizeof (String));
	row[0] = opcddb.cdmail;
	XbaeMatrixAddRows (wLbmail, pos, row, NULL, NULL, 1);
	XtFree ((char *)row);
	XbaeMatrixEnableRedisplay (wLbmail, True);
}

/*
 * Folgende Funktion wird aufgerufen wenn der Button '-' ueber der
 * linken Listbox gedrueckt wird. Die Funktion loescht die selektierte
 * Zeile aus der linken Listbox.
 */
void DeleteCDDBServer (Widget w, XtPointer tag, XtPointer data)
{
int r, c;

	XbaeMatrixFirstSelectedCell (wLbserver, &r, &c);

	if (r == -1)
	{
	   XmtDisplayError (w, "ErrorNoSelection",
		"Please select an entry first!");
	   return;
	}

	XbaeMatrixDisableRedisplay (wLbserver);
	XbaeMatrixDeleteRows (wLbserver, r, 1);
	XbaeMatrixEnableRedisplay (wLbserver, True);
}

/*
 * Folgende Funktion wird aufgerufen wenn der Button '-' ueber der
 * rechten Listbox gedrueckt wird. Die Funktion loescht die selektierte
 * Zeile aus der rechten Listbox.
 */
void DeleteEMAILDestination (Widget w, XtPointer tag, XtPointer data)
{
int r, c;

	XbaeMatrixFirstSelectedCell (wLbmail, &r, &c);

	if (r == -1)
	{
	   XmtDisplayError (w, "ErrorNoSelection",
		"Please select an entry first!");
	   return;
	}

	XbaeMatrixDisableRedisplay (wLbmail);
	XbaeMatrixDeleteRows (wLbmail, r, 1);
	XbaeMatrixEnableRedisplay (wLbmail, True);
}

/*
 * Die folgenden zwei Funktionen werden aufgerufen wenn der User in eine
 * der beiden Listboxen klickt. Als Folge davon wird der Inhalt der
 * angeklickten Zeile in die Eingabezeile kopiert.
 */
void opLbServerCallback (Widget w, XtPointer tag, XtPointer data)
{
XbaeMatrixEnterCellCallbackStruct *ec = (XbaeMatrixEnterCellCallbackStruct *)data;
String s;

	ec->doit = False;
	ec->map = False;
	XbaeMatrixDeselectAll (w);
	XbaeMatrixSelectRow (w, ec->row);
	XmtDialogGetDialogValues (OpCDDBWidget, &opcddb);
	s = XbaeMatrixGetCell (w, ec->row, 0);
	strcpy (opcddb.cdserver, s);
	XmtDialogSetDialogValues (OpCDDBWidget, &opcddb);
}

void opLbMailCallback (Widget w, XtPointer tag, XtPointer data)
{
XbaeMatrixEnterCellCallbackStruct *ec = (XbaeMatrixEnterCellCallbackStruct *)data;
String s;

	ec->doit = False;
	ec->map = False;
	XbaeMatrixDeselectAll (w);
	XbaeMatrixSelectRow (w, ec->row);
	XmtDialogGetDialogValues (OpCDDBWidget, &opcddb);
	s = XbaeMatrixGetCell (w, ec->row, 0);
	strcpy (opcddb.cdmail, s);
	XmtDialogSetDialogValues (OpCDDBWidget, &opcddb);
}

/*
 * Teilt eine Zeile der linken Listbox, welche eine CDDB-Serverdefinition
 * enthaelt, in ihre einzelnen Komponenten auf. Die Zeile ist wie folgt
 * aufgebaut:
 *    <Server Name> <Port> <Protokoll> [<Adresspfad>]
 * Der Adresspfad wird nur benoetigt wenn HTTP-Protokoll angewendet wird.
 */
int SplittServerLine (char *line, char *sname, int *sport, int *proto, char *addr)
{
char *p, *l;
int i = 0;

	if ((l = strdup (line)) == NULL)
	   return -1;

	if ((p = strtok (l, " ")) == NULL)
	   return -1;

	while (p)
	{
	   switch (i)
	   {
	      case 0: strcpy (sname, p); break;
	      case 1: *sport = atoi (p); break;

	      case 2: 
		 if (!strcasecmp (p, "CDDB") || !strcasecmp (p, "CDDBP"))
		    *proto = CDDB_MODE_CDDBP;
		 else if (!strcasecmp (p, "HTTP"))
		    *proto = CDDB_MODE_HTTP;
		 else if (!strcasecmp (p, "CDI"))
		    *proto = CDINDEX_MODE_HTTP;
		 else if (!strcasecmp (p, "COVR"))
		    *proto = COVERART_MODE_HTTP;
		 else
		    *proto =CDDB_MODE_CDDBP;
	      break;

	      case 3:
		 if (*p != '-')
		    strcpy (addr, p);
	   }

	   p = strtok (NULL, " ");
	   i++;
	}

	if (i < 2)
	   return -1;

	return 0;
}
