/*
 * File   : printu.c
 * Purpose: This file contains routines for creating a postscript file. 
 * Autor  : Andreas Theofilu
 *
 * Copyright (C) 1998 by Theos Soft.
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <Xmt/Xmt.h>
#include <Xmt/Create.h>
#include <Xmt/Converters.h>
#include <Xmt/Procedures.h>
#include <Xmt/Layout.h>
#include <Xmt/WidgetType.h>
#include <Xmt/Dialog.h>
#include <Xmt/Dialogs.h>

#include "printu.h"
#include "platten.h"

/*
 * Resourceliste zum Austausch von Parametern mit der Dialogbox.
 */
typedef struct
{
	char info[128];
} print_t;

XtResource print_resource[] = {
	{"info", "info", XmtRBuffer, XmtSizeOf (print_t, info),
	 XtOffsetOf (print_t, info), XtRImmediate, NULL}
};

/*
 * Einige globale Variablen
 */
FILE *prfs;
FILE *fdhead;
char Printer[128];
char sTmpFile[256], sHeadFile[256];
int Metric;
char sMetric[11];
int Pages;
int prIsFile;
print_t print;
char zeile1[128], zeile2[128];
Widget wPrint = NULL;
Boolean PrintCancel;

/*
 * Funktionsprototypen die nur in diesem Modul verwendet werden.
 */
void HeaderRest ();

int PsOpenPrint (char *pr, int toFile)
{
	sTmpFile[0] = sHeadFile[0] = 0;
	Metric = 0;
	Pages = 0;
	strcpy (sMetric, "inch");

	if (toFile)
	   prIsFile = 1;
	else
	   prIsFile = 0;

	if (pr != NULL)
	{
	   strncpy (Printer, pr, sizeof (Printer));
	   Printer[sizeof (Printer) - 1] = 0;
	}
	else
	   Printer[0] = 0;

	if (toFile)
	   sprintf (zeile1, "Schreibe in Datei: %s", Printer);
	else
	   sprintf (zeile1, "Drucke zu: %s", Printer);

	if (tmpnam (&sTmpFile[0]) == NULL)
	   return 0;		/* Boolean! */

	if ((prfs = fopen (sTmpFile, "w+")) == NULL)
	   return 0;

	if (tmpnam (sHeadFile) == NULL)
	{
	   fclose (prfs);
	   unlink (sTmpFile);
	   return 0;
	}

	if ((fdhead = fopen (sHeadFile, "w+")) == NULL)
	{
	   fclose (prfs);
	   unlink (sTmpFile);
	   return 0;
	}

	/*
	 * Erzeugen und anzeigen des Druckstatusfensters.
	 */
	PrintCancel = False;
	sprintf (print.info, "%s\nAktuelle Seite: 0", zeile1);

	if (!wPrint)
	{
	   GetResourcePath ("Prints");
	   wPrint = XmtBuildDialog (toplevel, "prints", print_resource,
			XtNumber (print_resource));

	   XmtAddDeleteCallback (XmtGetShell (wPrint), XmDO_NOTHING,
		StopPrintCallback, NULL);
	}

	XmtDialogDo (wPrint, &print);
	XmUpdateDisplay (wPrint);
	return 1;
}

void StopPrintCallback (Widget w, XtPointer tag, XtPointer data)
{
	XmUpdateDisplay (wPrint);
	PrintCancel = True;
}

Boolean GetPrintStatus ()
{
	XmUpdateDisplay (wPrint);
	return PrintCancel;
}

void HeaderRest ()
{
	fprintf (fdhead, "%%!PS-Adobe-2.0\n");
	fprintf (fdhead, "%%%%Creator: Theos Soft Report Generator\n");
	fprintf (fdhead, "%%%%DocumentData: Clean8Bit\n");
	fprintf (fdhead, "%%%%DocumentPaperSizes: a4\n");
	fprintf (fdhead, "%%%%Orientation: Portrait\n");
	fprintf (fdhead, "%%%%Title: Theos Soft Report\n");
	fprintf (fdhead, "%%%%BoundingBox: 0 0 596 842\n");
	fprintf (fdhead, "%%%%PageOrder: Ascend\n");
	fprintf (fdhead, "%%%%Pages: %d\n", Pages);
	fprintf (fdhead, "%%%%EndComments\n");
	fprintf (fdhead, "/inch {72 mul} def\n");
	fprintf (fdhead, "/milli {2.835 mul} def\n");
	fprintf (fdhead, "[ /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n");
	fprintf (fdhead, " /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n");
	fprintf (fdhead, " /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n");
	fprintf (fdhead, " /.notdef /.notdef /space /exclam /quotedbl /numbersign /dollar /percent /ampersand /quoteright\n");
	fprintf (fdhead, " /parenleft /parenright /asterisk /plus /comma /hyphen /period /slash /zero /one\n");
	fprintf (fdhead, " /two /three /four /five /six /seven /eight /nine /colon /semicolon\n");
	fprintf (fdhead, " /less /equal /greater /question /at /A /B /C /D /E\n");
	fprintf (fdhead, " /F /G /H /I /J /K /L /M /N /O\n");
	fprintf (fdhead, " /P /Q /R /S /T /U /V /W /X /Y\n");
	fprintf (fdhead, " /Z /bracketleft /backslash /bracketright /asciicircum /underscore /quoteleft /a /b /c\n");
	fprintf (fdhead, " /d /e /f /g /h /i /j /k /l /m\n");
	fprintf (fdhead, " /n /o /p /q /r /s /t /u /v /w\n");
	fprintf (fdhead, " /x /y /z /braceleft /bar /braceright /asciitilde /.notdef /.notdef /.notdef\n");
	fprintf (fdhead, " /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n");
	fprintf (fdhead, " /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n");
	fprintf (fdhead, " /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n");
	fprintf (fdhead, " /space /exclamdown /cent /sterling /currency /yen /brokenbar /section /dieresis /copyright\n");
	fprintf (fdhead, " /ordfeminine /guillemotleft /logicalnot /hyphen /registered /macron /degree /plusminus /twosuperior /threesuperior\n");
	fprintf (fdhead, " /acute /mu /paragraph /periodcentered /cedilla /onesuperior /ordmasculine /guillemotright /onequarter /onehalf\n");
	fprintf (fdhead, " /threequarters /questiondown /Agrave /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla\n");
	fprintf (fdhead, " /Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex /Idieresis /Eth /Ntilde\n");
	fprintf (fdhead, " /Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply /Oslash /Ugrave /Uacute /Ucircumflex\n");
	fprintf (fdhead, " /Udieresis /Yacute /Thorn /germandbls /agrave /aacute /acircumflex /atilde /adieresis /aring\n");
	fprintf (fdhead, " /ae /ccedilla /egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex /idieresis\n");
	fprintf (fdhead, " /eth /ntilde /ograve /oacute /ocircumflex /otilde /odieresis /divide /oslash /ugrave\n");
	fprintf (fdhead, " /uacute /ucircumflex /udieresis /yacute /thorn /ydieresis] /isolatin1encoding exch def\n");
	fprintf (fdhead, "/c { matrix currentmatrix currentpoint translate\n");
	fprintf (fdhead, "     3 1 roll scale newpath 0 0 1 0 360 arc setmatrix } bind def\n");
	fprintf (fdhead, "/F0\n");
	fprintf (fdhead, "    /Times-Roman findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f0 { /F0 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F1\n");
	fprintf (fdhead, "    /Times-Bold findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f1 { /F1 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F2\n");
	fprintf (fdhead, "    /Times-Italic findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f2 { /F2 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F3\n");
	fprintf (fdhead, "    /Times-BoldItalic findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f3 { /F3 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F4\n");
	fprintf (fdhead, "    /Courier findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f4 { /F4 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F5\n");
	fprintf (fdhead, "    /Courier-Bold findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f5 { /F5 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F6\n");
	fprintf (fdhead, "    /Courier-Oblique findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f6 { /F6 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F7\n");
	fprintf (fdhead, "    /Courier-BoldOblique findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f7 { /F7 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F8\n");
	fprintf (fdhead, "    /Helvetica findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f8 { /F8 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F9\n");
	fprintf (fdhead, "    /Helvetica-Bold findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f9 { /F9 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F10\n");
	fprintf (fdhead, "    /Helvetica-Italic findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f10 { /F10 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/F11\n");
	fprintf (fdhead, "    /Helvetica-BoldItalic findfont\n");
	fprintf (fdhead, "    dup length dict begin\n");
	fprintf (fdhead, "	{1 index /FID ne {def} {pop pop} ifelse} forall\n");
	fprintf (fdhead, "	/Encoding isolatin1encoding def\n");
	fprintf (fdhead, "    currentdict end\n");
	fprintf (fdhead, "definefont pop\n");
	fprintf (fdhead, "/f11 { /F11 findfont exch scalefont setfont } bind def\n");
	fprintf (fdhead, "/rhc {\n");
	fprintf (fdhead, "    {\n");
	fprintf (fdhead, "        currentfile read {\n");
	fprintf (fdhead, "	    dup 97 ge\n");
	fprintf (fdhead, "		{ 87 sub true exit }\n");
	fprintf (fdhead, "		{ dup 48 ge { 48 sub true exit } { pop } ifelse }\n");
	fprintf (fdhead, "	    ifelse\n");
	fprintf (fdhead, "	} {\n");
	fprintf (fdhead, "	    false\n");
	fprintf (fdhead, "	    exit\n");
	fprintf (fdhead, "	} ifelse\n");
	fprintf (fdhead, "    } loop\n");
	fprintf (fdhead, "} bind def\n\n");
/**	fprintf (fdhead, "/cvgray { % xtra_char npix cvgray - (string npix long)\n");
	fprintf (fdhead, "    dup string\n");
	fprintf (fdhead, "    0\n");
	fprintf (fdhead, "    {\n");
	fprintf (fdhead, "	rhc { cvr 4.784 mul } { exit } ifelse\n");
	fprintf (fdhead, "	rhc { cvr 9.392 mul } { exit } ifelse\n");
	fprintf (fdhead, "	rhc { cvr 1.824 mul } { exit } ifelse\n");
	fprintf (fdhead, "	add add cvi 3 copy put pop\n");
	fprintf (fdhead, "	1 add\n");
	fprintf (fdhead, "	dup 3 index ge { exit } if\n");
	fprintf (fdhead, "    } loop\n");
	fprintf (fdhead, "    pop\n");
	fprintf (fdhead, "    3 -1 roll 0 ne { rhc { pop } if } if\n");
	fprintf (fdhead, "    exch pop\n");
	fprintf (fdhead, "} bind def\n\n");
***/
	fprintf (fdhead, "/smartimage12rgb { %% w h b [matrix] smartimage12rgb -\n");
	fprintf (fdhead, "    /colorimage where {\n");
	fprintf (fdhead, "	pop\n");
	fprintf (fdhead, "	{ currentfile rowdata readhexstring pop }\n");
	fprintf (fdhead, "	false 3\n");
	fprintf (fdhead, "	colorimage\n");
	fprintf (fdhead, "    } {\n");
	fprintf (fdhead, "	exch pop 8 exch\n");
	fprintf (fdhead, "	3 index 12 mul 8 mod 0 ne { 1 } { 0 } ifelse\n");
	fprintf (fdhead, "	4 index\n");
	fprintf (fdhead, "	6 2 roll\n");
	fprintf (fdhead, "	{ 2 copy cvgray }\n");
	fprintf (fdhead, "	image\n");
	fprintf (fdhead, "	pop pop\n");
	fprintf (fdhead, "    } ifelse\n");
	fprintf (fdhead, "} def\n");
	fprintf (fdhead, "/cshow { dup stringwidth pop 2 div neg 0 rmoveto show } bind def\n");
	fprintf (fdhead, "/rshow { dup stringwidth pop neg 0 rmoveto show } bind def\n");
	fprintf (fdhead, "%%%%EndProlog\n");
}

void PsClosePrint ()
{
char prCommand[256];
char buf[16384];
int ret;

	HeaderRest ();
	fseek (prfs, 0L, SEEK_SET);

	while ((ret = fread (&buf[0], sizeof (char), sizeof (buf), prfs)) > 0)
	   fwrite (&buf[0], sizeof (char), ret, fdhead);

	fclose (prfs);
	fclose (fdhead);
	unlink (sTmpFile);

	if (prIsFile)
	{
	   sprintf (prCommand, "mv %s %s", sHeadFile, Printer);
	   system (prCommand);
	}
	else
	{
	   if (Printer[0])
	      sprintf (prCommand, "lpr -P%s %s", Printer, sHeadFile);
	   else
	      sprintf (prCommand, "lpr %s", sHeadFile);

	   system (prCommand);
	   unlink (sHeadFile);
	}

	prfs = NULL;
	fdhead = NULL;
	sTmpFile[0] = sHeadFile[0] = 0;
	Pages = 0;
	Printer[0] = 0;
	XtUnmanageChild (wPrint);
}

void PsSetMetric (int m)
{
	if (m < 0 || m > 2)
	   return;

	if (Metric != m)
	{
	   switch (m)
	   {
	      case PsINCH:
		 strcpy (sMetric, "inch");
	      break;

	      case PsPT:
		 strcpy (sMetric, "\0");
	      break;

	      case PsMILLIMETER:
		 strcpy (sMetric, "milli");
	      break;
	   }
	}

	Metric = m;
}

float PsMilliToPt (float m)
{
	return m * 2.835;
}

float PsInchToPt (float i)
{
	return i * 72.0;
}

void PsNewPath ()
{
	fprintf (prfs, "newpath\n");
}

void PsShowPage ()
{
	fprintf (prfs, "showpage\n");
	Pages++;

	sprintf (zeile2, "Aktuelle Seite: %d", Pages);
	sprintf (print.info, "%s\n%s", zeile1, zeile2);
	XmtDialogSetDialogValues (wPrint, &print);
	XmUpdateDisplay (wPrint);
}

int PsGetPage ()
{
	return Pages + 1;
}

void PsNewPage ()
{
	fprintf (prfs, "%%%%Page: %d %d\n", Pages + 1, Pages + 1);
}

void PsSetFont (char *fontname, int scale)
{
	if (fontname == NULL || !*fontname)
	   return;

	if (scale < 4 || scale > 120)
	   return;

	fprintf (prfs, "/%s findfont\n", fontname);
	fprintf (prfs, "%d scalefont setfont\n", scale);
}

void PsChooseFont (int fn, int scale)
{
	if (fn < 0 || fn > 11)
	   return;

	if (scale < 4 || scale > 1000)
	   return;

	fprintf (prfs, "%d f%d\n", scale, fn);
}

void PsSetPosition (float x, float y)
{
float ptx, pty;

	switch (Metric)
	{
	   case PsMILLIMETER:
	      ptx = x * 2.835;
	      pty = y * 2.835;
	   break;

	   case PsINCH:
	      ptx = x * 72.0;
	      pty = y * 72.0;
	   break;

	   default:
	      ptx = x;
	      pty = y;
	}

/**	fprintf (prfs, "%f %s %f %s moveto\n", x, sMetric, y, sMetric); **/
	fprintf (prfs, "%f %f moveto\n", ptx, pty);
}

void PsSetRelativePosition (float x, float y)
{
float ptx, pty;

	switch (Metric)
	{
	   case PsMILLIMETER:
	      ptx = x * 2.835;
	      pty = y * 2.835;
	   break;

	   case PsINCH:
	      ptx = x * 72.0;
	      pty = y * 72.0;
	   break;

	   default:
	      ptx = x;
	      pty = y;
	}

/**	fprintf (prfs, "%f %s %f %s rmoveto\n", x, sMetric, y, sMetric); **/
	fprintf (prfs, "%f %f rmoveto\n", ptx, pty);
}

void PsLineTo (float x, float y)
{
float ptx, pty;

	switch (Metric)
	{
	   case PsMILLIMETER:
	      ptx = x * 2.835;
	      pty = y * 2.835;
	   break;

	   case PsINCH:
	      ptx = x * 72.0;
	      pty = y * 72.0;
	   break;

	   default:
	      ptx = x;
	      pty = y;
	}

	fprintf (prfs, "%f %f rlineto\n", ptx, pty);
}

void PsDrawLine (float xs, float ys, float xt, float yt)
{
float ptxs, ptys, ptxt, ptyt;

	switch (Metric)
	{
	   case PsMILLIMETER:
	      ptxs = xs * 2.835;
	      ptys = ys * 2.835;
	      ptxt = xt * 2.835;
	      ptyt = yt * 2.835;
	   break;

	   case PsINCH:
	      ptxs = xs * 72.0;
	      ptys = ys * 72.0;
	      ptxt = xt * 72.0;
	      ptyt = yt * 72.0;
	   break;

	   default:
	      ptxs = xs;
	      ptys = ys;
	      ptxt = xt;
	      ptyt = yt;
	}

	fprintf (prfs, "%f %f moveto %f %f lineto\n", ptxs, ptys, ptxt, ptyt);
}

void PsDrawText (char *text)
{
char *p;

	if (!strlen (text))
	   return;

	p = text;
	fputs ("(", prfs);

	while (*p)
	{
	   switch (*p)
	   {
	      case '(': fputc ('\\', prfs); break;
	      case ')': fputc ('\\', prfs); break;
	   }

	   fputc (*p, prfs);
	   p++;
	}

	fputs (") show\n", prfs);
}

void PsSetLineWidth (int pt)
{
	if (pt < 1 || pt > 120)
	   return;

	fprintf (prfs, "%d setlinewidth\n", pt);
}

void PsSetGrayLevel (int gr)
{
float lv;

	if (gr < 0 || gr > 100)
	   return;

	lv = (float)gr / 100.0;
	fprintf (prfs, "%f setgray\n", lv);
}

void PsClosePath (int type)
{
char stype[22];

	switch (type)
	{
	   case 0: strcpy (stype, "closepath"); break;
	   case 1: strcpy (stype, "stroke"); break;
	   case 2: strcpy (stype, "fill"); break;
	}

	fprintf (prfs, "%s\n", stype);
}

void PsPushState ()
{
	fprintf (prfs, "gsave\n");
}

void PsPopState ()
{
	fprintf (prfs, "grestore\n");
}

